use rgb::Rgb;
use super::smart_leds_lib::rmt::FastWs2812Esp32Rmt;
use super::smart_leds_lib::StrideOutput;

use crate::task::{FixedSizeScheduler, Task};
use crate::buffers::StaticSurfacePool;

use super::Board;

pub struct Esp32Board<'a> {
    output: Option<<Self as Board>::Output>,
    surfaces: Option<StaticSurfacePool>,
    tasks: Option<[&'a mut dyn Task; 1]>
}

impl<'a> Board for Esp32Board<'a> {
    type Output = StrideOutput<[Rgb<u8>; 310], FastWs2812Esp32Rmt<'a>>;
    type Surfaces = StaticSurfacePool;
    type Scheduler = FixedSizeScheduler<0>;
    fn take() -> Self {
        let peripherals = esp_hal::init(esp_hal::Config::default());
        //esp_alloc::heap_allocator!(72 * 1024);

        const POWER_VOLTS : u32 = 5;
        const POWER_MA : u32 = 500;
        const MAX_POWER_MW : u32 = POWER_VOLTS * POWER_MA;
        let pins = peripherals.pins;


        Esp32Board { output: None, surfaces: None, tasks: None }
    }

    fn output(&mut self) -> Self::Output {
        self.output.take().unwrap()
    }

    fn surfaces(&mut self) -> Self::Surfaces {
        self.surfaces.take().unwrap()
    }

    fn system_tasks(&mut self) -> Self::Scheduler {
        FixedSizeScheduler::new([])
    }
}